function images = pco_adaptor_example_immediateaquire(imcount, timeout_s)
% This function records and returns a series of images using the immediate 
% triggering of the imaq adaptor.
%
%    [images] = pco_adaptor_example_immediateaquire(imcount, timeout_s)
%
% * Input parameters
%       imcount:        Number of images to acquire
%       timeout_s:      Number of seconds to wait before video input 
%                       timeout occurs
%
% * Output parameters
%       images:         Stack with acquired images
%
% - Set property values and trigger mode
% - Acquire images immediately after starting the videoinput device
% - Display images and reset the adaptor
%

%% Set default values for paramters that haven't been set
if(~exist('imcount','var'))
    imcount = 10;
end

if(~exist('timeout_s','var'))
    timeout_s = 120;
end

%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure camera and record images
%Create video input object
vid = videoinput(adaptorName, 0);

%Create adaptor source
src = getselectedsource(vid);

%Set timestamp mode
src.TMTimestampMode = 'BinaryAndAscii';

%Set logging to memory
vid.LoggingMode = 'memory';

%Set trigger mode
triggerconfig(vid, 'immediate');

%Set frames per trigger
vid.FramesPerTrigger = imcount;

%Set the Timeout property of video input object 'vid'
set(vid,'Timeout',timeout_s); 

%Start acquisition
start(vid);

%Read out all images and remove them from memory buffer
%Function blocks until the images have been aquired
images = getdata(vid);

%% Show images
%Show images in Video Viewer app
implay(images);

%Reset adaptor
imaqreset;
