function image = pco_adaptor_example_livepreview()
% This function turns on the live preview and stops it when the user 
% persses a key. After preview stopped, it extracts and shows a snapshot
% image.
%
%   [image] = pco_adaptor_example_livepreview()
%
% * Output parameter
%       image:      Single image snapshot
%
% - Start preview, stop preview, acquire image
% - Display single image snapshot and reset the adaptor
%

%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure camera and show live preview
%Create video input object
vid = videoinput(adaptorName, 0);

%Set logging to memory
vid.LoggingMode = 'memory';

%Start preview
preview(vid);

%Wait for keypress
pause;

%Stop preview
stoppreview(vid);

%Get single image
image = getsnapshot(vid);

%% Show image
%Show image
imshow(image);

%Reset adaptor
imaqreset;