function phasor = flim_referencing_phasor(phasorSample, phasorRef, ...
    tauRef, f)

%
% function phasor = flim_referencing_phasor(phasorSample, phasorRef, ...
%     tauRef, f)
%
% This function references a sample measurement by a reference measurement.
% The reference sample is assumed to be an ideal first-order lowpass
% system whose behaviour is characterized by its time constant tauRef at a
% specific modulation frequency f.
%
% Input parameters:
% phasorSample: 2D complex phasor image of sample measurement
% phasorRef:    2D complex phasor image of reference measurement
% tauRef:       Time constant of reference system (in seconds)
% f:            Modulation frequency (in Hertz)
%
% Return value:
% phasor: Referenced 2D complex phasor image
%

    phasor = (1 / (1 + i * 2 * pi * f * tauRef)) .* ...
             phasorSample ./ phasorRef;

end
